//
//
// hello.ftdi.44.echo.interrupt.c
//
// 115200 baud FTDI character echo, interrupt version
//
// set lfuse to 0x7E for 20 MHz xtal
//
// Neil Gershenfeld
// 12/8/10
//
// (c) Massachusetts Institute of Technology 2010
// Permission granted for experimental and personal use;
// license for commercial sale available from MIT.
//

#include <stdlib.h>
#include <stdio.h>

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
//#include <avr/pgmspace.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
#define bit_delay_time 8.5 // bit delay for 115200 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define char_delay() _delay_ms(10) // char delay

#define serial_port PORTA
#define serial_direction DDRA
#define serial_pins PINA
#define serial_pin_in (1 << PA0)
#define serial_pin_out (1 << PA1)
#define serial_interrupt (1 << PCIE0)
#define serial_interrupt_pin (1 << PCINT0)

#define max_buffer 25

volatile int *mode;

volatile char *chr;
//volatile int act = 0;

void get_char(volatile unsigned char *pins, unsigned char pin, char *rxbyte) {
   //
   // read character into rxbyte on pins pin
   //    assumes line driver (inverts bits)
   //
   *rxbyte = 0;
   while (pin_test(*pins,pin))
      //
      // wait for start bit
      //
      ;
   //
   // delay to middle of first data bit
   //
   half_bit_delay();
   bit_delay();
   //
   // unrolled loop to read data bits
   //
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 0);
   else
      *rxbyte |= (0 << 0);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 1);
   else
      *rxbyte |= (0 << 1);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 2);
   else
      *rxbyte |= (0 << 2);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 3);
   else
      *rxbyte |= (0 << 3);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 4);
   else
      *rxbyte |= (0 << 4);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 5);
   else
      *rxbyte |= (0 << 5);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 6);
   else
      *rxbyte |= (0 << 6);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 7);
   else
      *rxbyte |= (0 << 7);
   //
   // wait for stop bit
   //
   bit_delay();
   half_bit_delay();
   }

void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
   //
   // send character in txchar on port pin
   //    assumes line driver (inverts bits)
   //
   // start bit
   //
   clear(*port,pin);
   bit_delay();
   //
   // unrolled loop to write data bits
   //
   if bit_test(txchar,0)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,1)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,2)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,3)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,4)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,5)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,6)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,7)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   //
   // stop bit
   //
   set(*port,pin);
   bit_delay();
   }

void put_string(volatile unsigned char *port, unsigned char pin, char *str) {
   //
   // print a null-terminated string
   //
   static int index;
   index = 0;
   do {
      put_char(port, pin, str[index]);
      ++index;
      } while (str[index] != 0);
   }

int initialize_led_switch(void){
   DDRA |= (1<<PA3);		// LED is an output and located at PA3
   PORTA |= (1<<PA2);		// activate the internal resistor for the switch on PA2
}

int blinking_led(void) {
   // the LED is blinking in a fixed pattern
   PORTA |= (1<<PA3);
   _delay_us(10000);
   PORTA &= ~ (1<<PA3);
   _delay_us(10000);
}

int switching_led(void) {
   // the LED is switched on by pressing the button
   if ((PINA & (1<<PA2)) == 0) { 
      PORTA |= (1<<PA3);
   } else { 
      PORTA &= ~ (1<<PA3);
   }
   _delay_us(100000);
}

int set_mode(char c){
      char str[16];
      sprintf(mode, str, 10); 
      put_string(&serial_port, serial_pin_out, "Actual Mode: ");
      put_char(&serial_port, serial_pin_out, str);
      put_char(&serial_port, serial_pin_out, 10); // new line
      put_string(&serial_port, serial_pin_out, "You chose: ");
      put_char(&serial_port, serial_pin_out, c);
      put_char(&serial_port, serial_pin_out, 10); // new line
      if ( c == 's' ) {
         put_string(&serial_port, serial_pin_out, "Switching Mode");
         mode = 1;
      } else if ( c == 'b' ) {
         put_string(&serial_port, serial_pin_out, "Blinking Mode");
         mode = 2;
      } 
      else if ( c == 'f' ) {
         put_string(&serial_port, serial_pin_out, "LED OFF");
         mode = 3;
      } 
      else if ( c == 'n' ) {
         put_string(&serial_port, serial_pin_out, "LED ON");
         mode = 4;
      } 
      else {
         put_string(&serial_port, serial_pin_out, "Unknown Mode -- Using switching instead! ");
         put_char(&serial_port, serial_pin_out, 10); // new line
         put_string(&serial_port, serial_pin_out, "s\tSwitching Mode");
         put_char(&serial_port, serial_pin_out, 10); // new line
         put_string(&serial_port, serial_pin_out, "b\tBlinking Mode");
         put_char(&serial_port, serial_pin_out, 10); // new line
         put_string(&serial_port, serial_pin_out, "f\tLED OFF");
         put_char(&serial_port, serial_pin_out, 10); // new line
         put_string(&serial_port, serial_pin_out, "n\tLED ON");
         put_char(&serial_port, serial_pin_out, 10); // new line
         put_char(&serial_port, serial_pin_out, 10); // new line
         mode = 1;
      }
      put_char(&serial_port, serial_pin_out, 10); // new line
      put_char(&serial_port, serial_pin_out, 10); // new line
}



ISR(PCINT0_vect) {
   //
   // pin change interrupt handler
   //
   //static char buffer[max_buffer] = {0};
   //static int index;
   //act=0;
   cli();
   get_char(&serial_pins, serial_pin_in, &chr);
   set_mode(chr);
   sei();
/*
   put_string(&serial_port, serial_pin_out, "hello.ftdi.44.echo.interrupt.c: you typed \"");
   buffer[index++] = chr;
   if (index == (max_buffer-1))
      index = 0;
   put_string(&serial_port, serial_pin_out, buffer);
   put_char(&serial_port, serial_pin_out, '\"');
   put_char(&serial_port, serial_pin_out, 10); // new line
*/
   }

int main(void) {
   //
   // main
   //
   // set clock divider to /1
   //

   int index=0;

   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // initialize output pins
   //
   set(serial_port, serial_pin_out);
   output(serial_direction, serial_pin_out);

   initialize_led_switch();
   set_mode('s');
   //act=true;
   //
   // set up pin change interrupt on input pin
   //
   set(GIMSK, serial_interrupt);
   set (PCMSK0, serial_interrupt_pin);
   sei();
   //
   // main loop
   //
   while ( 1 ) {
      index +=1;
/*
      if (index < 128)
         set_mode('s');
      else
         set_mode('b');
*/
      if (index > 254)
         index = 0;
      //
      // wait for interrupt
      //
      //if ( act > 0) {
	      if ( mode == 1 )
		 switching_led();
	      // blinking mode
	      else if ( mode == 2 )
		 blinking_led();
	      // off
	      else if ( mode == 3 )
	      	PORTA &= ~ (1<<PA3);
	      // on
	      else if ( mode == 4 )
		 PORTA |= (1<<PA3);
	      else {

         put_string(&serial_port, serial_pin_out, " ##LOOP## Unknown Mode : ");
         put_char(&serial_port, serial_pin_out, mode);
         put_char(&serial_port, serial_pin_out, 10); // new line
         }
      }
   }
